@@ -11,6 +11,7 @@
 # under the License.
 
 import functools
+import typing as ty
 import urllib
 from urllib.parse import urlparse
 
@@ -19,8 +20,9 @@ try:
 
     JSONDecodeError = simplejson.scanner.JSONDecodeError
 except ImportError:
-    JSONDecodeError = ValueError
+    JSONDecodeError = ValueError  # type: ignore
 import iso8601
+import jmespath
 from keystoneauth1 import adapter
 
 from openstack import _log
@@ -28,6 +30,9 @@ from openstack import exceptions
 from openstack import resource
 
 
+ResourceType = ty.TypeVar('ResourceType', bound=resource.Resource)
+
+
 # The _check_resource decorator is used on Proxy methods to ensure that
 # the `actual` argument is in fact the type of the `expected` argument.
 # It does so under two cases:
@@ -76,6 +81,12 @@ class Proxy(adapter.Adapter):
     ``<service-type>_status_code_retries``.
     """
 
+    _resource_registry: ty.Dict[str, ty.Type[resource.Resource]] = {}
+    """Registry of the supported resourses.
+
+    Dictionary of resource names (key) types (value).
+    """
+
     def __init__(
         self,
         session,
@@ -86,14 +97,14 @@ class Proxy(adapter.Adapter):
         influxdb_config=None,
         influxdb_client=None,
         *args,
-        **kwargs
+        **kwargs,
     ):
         # NOTE(dtantsur): keystoneauth defaults retriable_status_codes to None,
         # override it with a class-level value.
         kwargs.setdefault(
             'retriable_status_codes', self.retriable_status_codes
         )
-        super(Proxy, self).__init__(session=session, *args, **kwargs)
+        super().__init__(session=session, *args, **kwargs)
         self._statsd_client = statsd_client
         self._statsd_prefix = statsd_prefix
         self._prometheus_counter = prometheus_counter
@@ -130,7 +141,7 @@ class Proxy(adapter.Adapter):
         connect_retries=1,
         global_request_id=None,
         *args,
-        **kwargs
+        **kwargs,
     ):
         conn = self._get_connection()
         if not global_request_id:
@@ -159,14 +170,14 @@ class Proxy(adapter.Adapter):
                 # Get from cache or execute and cache
                 response = conn._cache.get_or_create(
                     key=key,
-                    creator=super(Proxy, self).request,
+                    creator=super().request,
                     creator_args=(
                         [url, method],
                         dict(
                             connect_retries=connect_retries,
                             raise_exc=raise_exc,
                             global_request_id=global_request_id,
-                            **kwargs
+                            **kwargs,
                         ),
                     ),
                     expiration_time=expiration_time,
@@ -176,13 +187,13 @@ class Proxy(adapter.Adapter):
                 # asked for cache bypass
                 self._invalidate_cache(conn, key_prefix)
                 # Pass through the API request bypassing cache
-                response = super(Proxy, self).request(
+                response = super().request(
                     url,
                     method,
                     connect_retries=connect_retries,
                     raise_exc=raise_exc,
                     global_request_id=global_request_id,
-                    **kwargs
+                    **kwargs,
                 )
 
             for h in response.history:
@@ -406,12 +417,6 @@ class Proxy(adapter.Adapter):
         except Exception:
             self.log.exception('Error writing statistics to InfluxDB')
 
-    def _version_matches(self, version):
-        api_version = self.get_api_major_version()
-        if api_version:
-            return api_version[0] == version
-        return False
-
     def _get_connection(self):
         """Get the Connection object associated with this Proxy.
 
@@ -423,14 +428,16 @@ class Proxy(adapter.Adapter):
             self, '_connection', getattr(self.session, '_sdk_connection', None)
         )
 
-    def _get_resource(self, resource_type, value, **attrs):
+    def _get_resource(
+        self, resource_type: ty.Type[ResourceType], value, **attrs
+    ) -> ResourceType:
         """Get a resource object to work on
 
         :param resource_type: The type of resource to operate on. This should
             be a subclass of :class:`~openstack.resource.Resource` with a
             ``from_id`` method.
         :param value: The ID of a resource or an object of ``resource_type``
-            class if using an existing instance, or ``munch.Munch``,
+            class if using an existing instance, or ``utils.Munch``,
             or None to create a new instance.
         :param attrs: A dict containing arguments for forming the request
             URL, if needed.
@@ -469,7 +476,13 @@ class Proxy(adapter.Adapter):
             value = resource.Resource._get_id(parent)
         return value
 
-    def _find(self, resource_type, name_or_id, ignore_missing=True, **attrs):
+    def _find(
+        self,
+        resource_type: ty.Type[ResourceType],
+        name_or_id,
+        ignore_missing=True,
+        **attrs,
+    ) -> ty.Optional[ResourceType]:
         """Find a resource
 
         :param name_or_id: The name or ID of a resource to find.
@@ -489,7 +502,13 @@ class Proxy(adapter.Adapter):
         )
 
     @_check_resource(strict=False)
-    def _delete(self, resource_type, value, ignore_missing=True, **attrs):
+    def _delete(
+        self,
+        resource_type: ty.Type[ResourceType],
+        value,
+        ignore_missing=True,
+        **attrs,
+    ) -> ty.Optional[ResourceType]:
         """Delete a resource
 
         :param resource_type: The type of resource to delete. This should
@@ -525,7 +544,13 @@ class Proxy(adapter.Adapter):
         return rv
 
     @_check_resource(strict=False)
-    def _update(self, resource_type, value, base_path=None, **attrs):
+    def _update(
+        self,
+        resource_type: ty.Type[ResourceType],
+        value,
+        base_path=None,
+        **attrs,
+    ) -> ResourceType:
         """Update a resource
 
         :param resource_type: The type of resource to update.
@@ -549,7 +574,12 @@ class Proxy(adapter.Adapter):
         res = self._get_resource(resource_type, value, **attrs)
         return res.commit(self, base_path=base_path)
 
-    def _create(self, resource_type, base_path=None, **attrs):
+    def _create(
+        self,
+        resource_type: ty.Type[ResourceType],
+        base_path=None,
+        **attrs,
+    ) -> ResourceType:
         """Create a resource from attributes
 
         :param resource_type: The type of resource to create.
@@ -573,7 +603,12 @@ class Proxy(adapter.Adapter):
         res = resource_type.new(connection=conn, **attrs)
         return res.create(self, base_path=base_path)
 
-    def _bulk_create(self, resource_type, data, base_path=None):
+    def _bulk_create(
+        self,
+        resource_type: ty.Type[ResourceType],
+        data,
+        base_path=None,
+    ) -> ty.Generator[ResourceType, None, None]:
         """Create a resource from attributes
 
         :param resource_type: The type of resource to create.
@@ -596,13 +631,13 @@ class Proxy(adapter.Adapter):
     @_check_resource(strict=False)
     def _get(
         self,
-        resource_type,
+        resource_type: ty.Type[ResourceType],
         value=None,
         requires_id=True,
         base_path=None,
         skip_cache=False,
-        **attrs
-    ):
+        **attrs,
+    ) -> ResourceType:
         """Fetch a resource
 
         :param resource_type: The type of resource to get.
@@ -637,7 +672,14 @@ class Proxy(adapter.Adapter):
             ),
         )
 
-    def _list(self, resource_type, paginated=True, base_path=None, **attrs):
+    def _list(
+        self,
+        resource_type: ty.Type[ResourceType],
+        paginated=True,
+        base_path=None,
+        jmespath_filters=None,
+        **attrs,
+    ) -> ty.Generator[ResourceType, None, None]:
         """List a resource
 
         :param resource_type: The type of resource to list. This should
@@ -650,6 +692,9 @@ class Proxy(adapter.Adapter):
         :param str base_path: Base part of the URI for listing resources, if
             different from
             :data:`~openstack.resource.Resource.base_path`.
+        :param str jmespath_filters: A string containing a jmespath expression
+            for further filtering.
+
         :param dict attrs: Attributes to be passed onto the
             :meth:`~openstack.resource.Resource.list` method. These should
             correspond to either :class:`~openstack.resource.URI` values
@@ -660,11 +705,22 @@ class Proxy(adapter.Adapter):
             :class:`~openstack.resource.Resource` that doesn't match
             the ``resource_type``.
         """
-        return resource_type.list(
+        data = resource_type.list(
             self, paginated=paginated, base_path=base_path, **attrs
         )
 
-    def _head(self, resource_type, value=None, base_path=None, **attrs):
+        if jmespath_filters and isinstance(jmespath_filters, str):
+            return jmespath.search(jmespath_filters, data)
+
+        return data
+
+    def _head(
+        self,
+        resource_type: ty.Type[ResourceType],
+        value=None,
+        base_path=None,
+        **attrs,
+    ) -> ResourceType:
         """Retrieve a resource's header
 
         :param resource_type: The type of resource to retrieve.
@@ -697,6 +753,7 @@ class Proxy(adapter.Adapter):
         identified_resources=None,
         filters=None,
         resource_evaluation_fn=None,
+        skip_resources=None,
     ):
         return None
 
@@ -775,7 +832,23 @@ class Proxy(adapter.Adapter):
         else:
             return False
 
+    def should_skip_resource_cleanup(self, resource=None, skip_resources=None):
+        if resource is None or skip_resources is None:
+            return False
 
+        resource_name = f"{self.service_type.replace('-', '_')}.{resource}"
+
+        if resource_name in skip_resources:
+            self.log.debug(
+                f"Skipping resource {resource_name} " "in project cleanup"
+            )
+            return True
+
+        return False
+
+
+# TODO(stephenfin): Remove this and all users. Use of this generally indicates
+# a missing Resource type.
 def _json_response(response, result_key=None, error_message=None):
     """Temporary method to use to bridge from ShadeAdapter to SDK calls."""
     exceptions.raise_from_response(response, error_message=error_message)
@@ -793,11 +866,3 @@ def _json_response(response, result_key=None, error_message=None):
     except JSONDecodeError:
         return response
     return result_json
-
-
-class _ShadeAdapter(Proxy):
-    """Wrapper for shade methods that expect json unpacking."""
-
-    def request(self, url, method, error_message=None, **kwargs):
-        response = super(_ShadeAdapter, self).request(url, method, **kwargs)
-        return _json_response(response, error_message=error_message)
